#! /bin/zsh

# We use zsh here to support floats (not supported in bash)

# This script produces a whole stack of subdirectories,
# for a range of values of c centered on 4 with 8 up/down factors of 2,
# and momenta in multiples of kF/4 from 1 to 16.

# For each combination c & k, the LiebLin_Catalogue_Fixed_c_k_Nscaling
# executable is invoked, which computes up to the highest N possible in view of time.


if [[ $# -ne 4 ]]; then
    echo "Arguments needed: whichDSF, kBT, target_sumrule, nr_minutes (for each c, k combination)."
    exit 1
fi

if [[ $1 != 'd' && $1 != 'g' && $1 != 'o' ]]; then
    echo "Only the d, g and o scanning options are implemented."
    exit 1
fi

if [[ $2 -lt 0 ]]; then
    echo "Temperature kBT must be > 0."
    exit 1
fi

if [[ $3 -lt 0 || $3 -gt 1 ]]; then
    echo "Requirement: 0 < target_sumrule < 1."
    exit 1
fi

if [[ $4 -lt 1 ]]; then
    echo "Please give a positive nr_minutes."
    exit 1
fi

zmodload zsh/datetime

whichDSF=$1
kBT=$2
target_sumrule=$3
nr_minutes=$4

correlator='rho-rho'
if [[ $whichDSF == 'o' ]]; then
    correlator='psidag-psi'
elif [[ $whichDSF == 'g' ]]; then
    correlator='psi-psidag'
fi

mkdir -p logs
logfile='logs/run_catalogue_k_fixed_'$whichDSF'_kBT_'$kBT'_sr_'$target_sumrule'_nr_minutes_'$nr_minutes'_'$(date '+%Y-%m-%d-%Hh%M')'.log'
touch $logfile

clist=(1024 512 256 128 64 32 16 8 4 2 1 0.5 0.25 0.125 0.0625 0.03125 0.015625)

basedir="$(pwd)"


for c in $clist
do
    for nk in {1..16}
    do
	echo '** Starting run for c =' $c', kfact = '$nk | tee -a $logfile
	dir='Lieb-Liniger/'$correlator'/T_'$kBT'/c_fixed/c_'$c'/k_fixed/k_'${(l:2::0:)nk}'kFo4/sr_'$target_sumrule'/prelim'
	mkdir -p $dir
	cd $dir
	LiebLin_Catalogue_Fixed_c_k_Nscaling $whichDSF $c $nk $kBT $target_sumrule 0 $nr_minutes | tee -a $basedir/$logfile
	cd $basedir
	echo '** Successfully completed run for c =' $c', kfact = '$nk'.\n' | tee -a $logfile
    done
done

echo '\n\n*** Successfully completed build_LiebLin_catalogue_k_fixed on '$(date '+%Y-%m-%d-%Hh%M') | tee -a $logfile
