#! /bin/zsh

# For given N (L) this
# computes the smoothened dsfs and produces the animated figure
# for a `c_scan_N_fixed` run.

if [[ $# -ne 6 ]]; then
    echo "Arguments needed: whichDSF, kBT, target_sumrule, N, nkmax (max momentum in units of kF/4), width."
    exit 1
fi

if [[ $1 != 'd' && $1 != 'g' && $1 != 'o' ]]; then
    echo "Only the d, g and o scanning options are implemented."
    exit 1
fi

if [[ $2 -lt 0 ]]; then
    echo "Temperature kBT must be > 0."
    exit 1
fi

if [[ $3 -lt 0 || $3 -gt 1 ]]; then
    echo "Requirement: 0 < target_sumrule < 1."
    exit 1
fi

if [[ $4 -lt 0 ]]; then
    echo "N must be > 0."
    exit 1
fi

if [[ $5 -lt 0 ]]; then
    echo "nkmax must be > 0."
    exit 1
fi

if [[ $6 -lt 0 ]]; then
    echo "width must be > 0."
    exit 1
fi


whichDSF=$1
kBT=$2
target_sumrule=$3
N=$4
nkmax=$5
width=$6

correlator='rho-rho'
if [[ $whichDSF == 'o' ]]; then
    correlator='psidag-psi'
elif [[ $whichDSF == 'g' ]]; then
    correlator='psi-psidag'
fi

# store location for these runs
dirstore='Lieb-Liniger/'$correlator'/T_'$kBT'/c_scan_N_fixed/k_0_to_'${(l:2::0:)nkmax}'kFo4/sr_'$target_sumrule'/store'

basedir="$(pwd)"

# Now create the animation
echo '** Animating c_scan_N_fixed for N = '$N'...'
dirplots=$dirstore'/plots/N_'$N
mkdir -p $dirplots
cd $dirplots
c_scan_N_fixed_animation.py $N $width
cd $basedir
echo '** Successfully animated c_scan_N_fixed for N = '$N'.\n'
